@@ -8,21 +8,22 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
+;; XEmacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with XEmacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
-;;; (Mostly) Synched up with: FSF 20.2
+;;; (Mostly) Synched up with: FSF 20.3
 ;;; XEmacs has extra function `diary-countdown'
 
 ;;; Commentary:
@@ -74,7 +75,7 @@ in the displayed three-month calendar."
           (error "Diary file is not readable!"))
       (error "You don't have a diary file!"))))
 
-(defun view-other-diary-entries (arg diary-file)
+(defun view-other-diary-entries (arg d-file)
   "Prepare and display buffer of diary entries from an alternative diary file.
 Prompts for a file name and searches that file for entries that match ARG
 days starting with the date indicated by the cursor position in the displayed
@@ -83,9 +84,9 @@ three-month calendar."
    (list (cond ((null current-prefix-arg) 1)
                ((listp current-prefix-arg) (car current-prefix-arg))
                (t current-prefix-arg))
-         (setq diary-file (read-file-name "Enter diary file name: "
-                                          default-directory nil t))))
-  (view-diary-entries arg))
+         (read-file-name "Enter diary file name: " default-directory nil t)))
+  (let ((diary-file d-file))
+    (view-diary-entries arg)))
 
 (autoload 'check-calendar-holidays "holidays"
   "Check the list of holidays for any that occur on DATE.
@@ -200,6 +201,7 @@ syntax of `*' changed to be a word constituent.")
 
 (modify-syntax-entry ?* "w" diary-syntax-table)
 
+;;;###autoload
 (defun list-diary-entries (date number)
   "Create and display a buffer containing the relevant lines in diary-file.
 The arguments are DATE and NUMBER; the entries selected are those
@@ -207,9 +209,11 @@ for NUMBER days starting with date DATE.  The other entries are hidden
 using selective display.
 
 Returns a list of all relevant diary entries found, if any, in order by date.
-The list entries have the form ((month day year) string).  If the variable
-`diary-list-include-blanks' is t, this list includes a dummy diary entry
-\(consisting of the empty string) for a date with no diary entries.
+The list entries have the form ((month day year) string specifier) where
+\(month day year) is the date of the entry, string is the entry text, and
+specifier is the applicability.  If the variable `diary-list-include-blanks'
+is t, this list includes a dummy diary entry consisting of the empty string)
+for a date with no diary entries.
 
 After the list is prepared, the hooks `nongregorian-diary-listing-hook',
 `list-diary-entries-hook', `diary-display-hook', and `diary-hook' are run.
@@ -322,13 +326,15 @@ These hooks have the following distinct roles:
                              (add-to-diary-list
                                date 
 			       (buffer-substring-no-properties 
-				entry-start (point)))))))
+                               entry-start (point))
+                              (buffer-substring-no-properties
+                               (1+ date-start) (1- entry-start)))))))
                      (setq d (cdr d)))
                    (or entry-found
                        (not diary-list-include-blanks)
                        (setq diary-entries-list 
                              (append diary-entries-list
-                                     (list (list date "")))))
+                                     (list (list date "" "")))))
                    (setq date
                          (calendar-gregorian-from-absolute
                            (1+ (calendar-absolute-from-gregorian date))))
@@ -581,6 +587,68 @@ is created."
           (error "Your diary file is not readable!"))
       (error "You don't have a diary file!"))))
 
+
+
+(defcustom diary-mail-addr 
+  (if (boundp 'user-mail-address) user-mail-address nil)
+  "*Email address that `diary-mail-entries' will send email to."
+  :group 'diary
+  :type '(choice string (const nil))
+  :version "20.3")
+
+(defcustom diary-mail-days 7
+  "*Number of days for `diary-mail-entries' to check."
+  :group 'diary
+  :type 'integer
+  :version "20.3")
+
+;;;###autoload
+(defun diary-mail-entries (&optional ndays)
+  "Send a mail message showing diary entries for next NDAYS days.
+If no prefix argument is given, NDAYS is set to `diary-mail-days'.
+
+You can call `diary-mail-entries' every night using an at/cron job.
+For example, this script will run the program at 2am daily.  Since
+`emacs -batch' does not load your `.emacs' file, you must ensure that
+all relevant variables are set, as done here.
+
+#!/bin/sh
+# diary-rem.sh -- repeatedly run the Emacs diary-reminder
+emacs -batch \\
+-eval \"(setq diary-mail-days 3 \\
+             european-calendar-style t \\
+             diary-mail-addr \\\"user@host.name\\\" )\" \\
+-l diary-lib -f diary-mail-entries 
+at -f diary-rem.sh 0200 tomorrow
+
+You may have to tweak the syntax of the `at' command to suit your
+system.  Alternatively, you can specify a cron entry:
+0 1 * * * diary-rem.sh
+to run it every morning at 1am."
+  (interactive "p")
+  (let ((text nil)
+       ;; Use the fancy-diary-display as it doesn't hide rest of
+       ;; diary file with ^M characters.  It also looks nicer.
+       (diary-display-hook 'fancy-diary-display))      
+    (if (not current-prefix-arg)
+       (setq ndays diary-mail-days))
+    (calendar)
+    (view-diary-entries ndays)
+    (set-buffer "*Fancy Diary Entries*")
+    (setq text (buffer-substring (point-min) (point-max)))
+
+    ;; Now send text as a mail message.
+    (mail)
+    (mail-to)
+    (insert diary-mail-addr)
+    (mail-subject)
+    (insert "Diary entries generated ")
+    (insert (format-time-string "%a %d %b %Y" (current-time)))
+    (mail-text)
+    (insert text)
+    (mail-send-and-exit nil)
+    (exit-calendar)))
+
 (defun diary-name-pattern (string-array &optional fullname)
   "Convert an STRING-ARRAY, an array of strings to a pattern.
 The pattern will match any of the strings, either entirely or abbreviated
@@ -879,11 +947,22 @@ A value of 0 in any position of the pattern is a wildcard."
            (< (diary-entry-time (car (cdr e1)))
               (diary-entry-time (car (cdr e2)))))))
 
+(defcustom diary-unknown-time
+  -9999
+  "*Value returned by diary-entry-time when no time is found.
+The default value -9999 causes entries with no recognizable time to be placed
+before those with times; 9999 would place entries with no recognizable time
+after those with times."
+  :type 'integer
+  :group 'diary
+  :version "20.3") 
+
 (defun diary-entry-time (s)
-  "Time at the beginning of the string S in a military-style integer.
-For example, returns 1325 for 1:25pm.  Returns -9999 if no time is recognized.
-The recognized forms are XXXX or X:XX or XX:XX (military time), XXam or XXpm,
-and XX:XXam or XX:XXpm."
+  "Time at the beginning of the string S in a military-style integer.  For
+example, returns 1325 for 1:25pm.  Returns `diary-unknown-time' (default value
+-9999) if no time is recognized.  The recognized forms are XXXX, X:XX, or
+XX:XX (military time), and XXam, XXAM, XXpm, XXPM, XX:XXam, XX:XXAM XX:XXpm,
+or XX:XXPM."
   (let ((case-fold-search nil))
   (cond ((string-match;; Military time  
           "^[ \t]*\\([0-9]?[0-9]\\):?\\([0-9][0-9]\\)\\(\\>\\|[^ap]\\)" s)
@@ -905,7 +984,7 @@ and XX:XXam or XX:XXpm."
             (string-to-int (substring s (match-beginning 2) (match-end 2)))
              (if (equal ?a (downcase (aref s (match-beginning 3))))
                 0 1200)))
-        (t -9999))));; Unrecognizable
+        (t diary-unknown-time))));; Unrecognizable
 
 (defun list-sexp-diary-entries (date)
   "Add sexp entries for DATE from the diary file to `diary-entries-list'.
@@ -1077,6 +1156,7 @@ best if they are nonmarking."
       (let ((sexp-start (point))
             (sexp)
             (entry)
+	    (specifier)
             (entry-start)
             (line-start))
         (forward-sexp)
@@ -1084,6 +1164,8 @@ best if they are nonmarking."
         (save-excursion
           (re-search-backward "\^M\\|\n\\|\\`")
           (setq line-start (point)))
+        (setq specifier
+              (buffer-substring-no-properties (1+ line-start) (point)))
         (forward-char 1)
         (if (and (or (char-equal (preceding-char) ?\^M)
                      (char-equal (preceding-char) ?\n))
@@ -1102,7 +1184,7 @@ best if they are nonmarking."
         (let ((diary-entry (diary-sexp-entry sexp entry date)))
           (if diary-entry
               (subst-char-in-region line-start (point) ?\^M ?\n t))
-          (add-to-diary-list date diary-entry)
+          (add-to-diary-list date diary-entry specifier)
           (setq entry-found (or entry-found diary-entry)))))
     entry-found))
 
@@ -1355,12 +1437,12 @@ a marking or nonmarking one."
         (and (or (not marking-diary-entries) marking-diary-entry)
              (eval sexp)))))
 
-(defun add-to-diary-list (date string)
-  "Add the entry (DATE STRING) to `diary-entries-list'.
+(defun add-to-diary-list (date string specifier)
+  "Add the entry (DATE STRING SPECIFIER) to `diary-entries-list'.
 Do nothing if DATE or STRING is nil."
   (and date string
        (setq diary-entries-list 
-             (append diary-entries-list (list (list date string))))))
+             (append diary-entries-list (list (list date string specifier))))))
 
 (defun make-diary-entry (string &optional nonmarking file)
   "Insert a diary entry STRING which may be NONMARKING in FILE.
